'use strict'
/**
 * @class Scenarioのメインとなるクラスです。
 * @classdesc Main of scenario
 * @constructor
 */
var Scenario = function () {
    /**
     * 試験が開始しているフラグ
     * @property {bool}
     */
    this.isStart = false;
    /**
     * 試験が中断しているフラグ
     * @property {bool}
     */
    this.isPause = false;
    /**
     * ポリシーの集まり
     * @property {Policies} - Policy
     */
    this.policies = new Policies();
    /**
     * @property {ScenarioForm}
     */
    let scenario_form = new ScenarioForm();
    /**
     *  試験を開始する関数
     */
    Scenario.prototype.Start = function () {
        this.policies.Start();
    }
    /**
     *  試験を中断する関数
     */
    Scenario.prototype.Pause = function () {
        this.policies.Pause();
    }
    /**
     *  ポリシー結果を初期化する関数
     */
    Scenario.prototype.Reset = function () {
        this.policies.DelResult();
    }
    /**
     * 試験ををリトライする関数
     * @param {number} num 試験番号(0から始まる)
     */
    Scenario.prototype.Retry = function (num) {
        scenario_form.OneRetry(num);
        this.policies.Retry(num);
    }
    /**
     *  試験を中止する関数
     */
    Scenario.prototype.ClearPolicies = function () {
        scenario_form.ClearPolicies();
        scenario_form.ClearScenarioBoxes();
        scenario_form.DefaultMessage();
        this.policies.Clear();
    }
    /**
     *  試験結果を出力する関数
     */
    Scenario.prototype.ExportResult = function () {
        let fs = require('fs');
        const electron = require('electron');
        const remote = electron.remote;
        const dialog = remote.dialog;
        const browserWindow = remote.BrowserWindow;
        saveNewFile();
        function writeFile(path, data) {
            fs.writeFile(path, data, function (error) {
                if (error != null) {
                    alert('error : ' + error);
                    return;
                }
            });
        }
        /**@inner */
        function saveNewFile() {
            var win = browserWindow.getFocusedWindow();
            dialog.showSaveDialog(
                win,
                {
                    properties: ['openFile'],
                    filters: [
                        {
                            name: 'Documents',
                            extensions: ['csv']
                        }
                    ]
                },
                // セーブ用ダイアログが閉じられた後のコールバック関数
                function (fileName) {
                    if (fileName) {
                        //テーブル->csv->配列へ保存->"Retry"を消去->csv
                        var data = $('#scenario-table').table2CSV();
                        //console.info("table convert csv...")
                        var array = data.split('\n');
                        var rows = [];
                        for (var i in array) {
                            var row = array[i];
                            var row_split = row.split(",");
                            row_split.splice(9, 1);
                            rows.push(row_split);
                        }
                        var csv = rows.join("\n");
                        var currentPath = fileName;
                        csv = '\ufeff'+csv;// 'feff'を付けるとBOM UTF8にする。
                        writeFile(currentPath,csv,'utf8');
                        //console.info("convert done")
                    }
                }
            );
        }
    }
    /**
     *  CSVからポリシーを読む込む関数
     */
    Scenario.prototype.ReadPolicyFile = function () {
        this.policies.ReadPolicyFile();
    }
    /**
     * Pauseフラグを返す
     * @return {bool} isPause
     */
    Scenario.prototype.GetPauseFlag = function () {
        return this.isPause;
    }
    /**
     * Startフラグを返す
     * @return {bool} isStart
     */
    Scenario.prototype.GetStartFlag = function () {
        return this.isStart;
    }
    /**
     * 試験結果をフィルターする。
     */
    Scenario.prototype.Filter = function (filter_name) {
        this.policies.scenario_form.Filter(filter_name, this.policies.policies.length);
    }
}

/**
 * @class 複数のポリシーを操作するためのクラスです。
 * @classdesc ポリシーの集まり
 * @constructor
 */
var Policies = function () {
    /**
     * passの合計
     * @property {num}
     */
    this.passSum = 0;
    /**
     * 現在のpassの数
     * @property {num}
     */
    this.passCount = 0;
    /**
     * dropの合計
     * @property {num}
     */
    this.dropSum = 0;
    /**
     * 現在のdropの数
     * type {num}
     */
    this.dropCount = 0;
    /**
     * 現在のerrorの数
     * @property {num}
     */
    this.errorCount = 0;
    /**
     * 現在のmismatの数
     * @property {num}
     */
    this.mismatchCount = 0;
    /**
     * エラーメッセージ
     * @property {string}
     */
    this.error_msg = "";
    /**
     * @property {bool}
     */
    this.isInvalid = false;
    /**
     * 試験が開始しているフラグ
     * @property {bool}
     */
    this.isStart = false;
    /**
     * 試験が中断しているフラグ
     * @property {bool}
     */
    this.isPause = false;
    /**
     *　DOMに登録するテーブルのHTMLの文字列
     * @property {string}
     */
    this.table_data = "";
    /**
     * policyインスタンスの集合
     * @property {Object}
     */
    this.policies = [];
    /**
     * 試験範囲(番号)
     * @property {Object}
     * @example {"begin":0,"end":0}
     */
    this.send_area = {};
    /**
     * ScenarioFormのインスタンス
     * @property {ScenarioForm}
     */
    this.scenario_form = new ScenarioForm();
    /**
     * フィルターされたポリシー番号の配列
     * @property {Object}
     */
    this.sendPolicyArray = [];
    /**
     * 送信カウント
     * @property {int}
     */
    this.sendCount = 0;
    /**
     * filterフラグ
     * @property {bool}
     */
    this.isFilter = false;
    /**
     * CSVからデータを読み込み関数
     */
    Policies.prototype.ReadPolicyFile = function () {
        let that = this;
        if ($('#scenario-upload').val() === "") {
            return;
        }
        let element_file = document.getElementById("scenario-upload");
        let file_list = element_file.files;
        let file = file_list[0];
        this.scenario_form.LoadCSV();
        if(!isCSV(file.name)){
            this.error_msg = "This file isn't CSV file."
            that.ShowError();
            return false;
        }
        setTimeout(function () {
            let expected_results = [];
            let file_reader = new FileReader();
            file_reader.readAsArrayBuffer(file);//非同期
            file_reader.onload = function (e) {//スコープ内の同期を保証
                let unicode_string = ConvertString(e.target.result);//Unicode変換された文字列を取得
                let csv_data = $.simple_csv(unicode_string);  // CSVファイルの内容を取得
                console.info("reading file...");
                for (let i = 1; i < csv_data.length; i++) { //1行目はヘッダなので飛ばす
                    let policy = new Policy();//ポリシーインスタンスを生成
                    policy.setCSVData(csv_data[i]);//インスタンスにcsvから値をセット
                    policy.Validate();//validateを実行する
                    expected_results.push(policy.getExpectResult());//期待する結果を抽出
                    that.policies.push(policy);//1ポリシーを保存
                    console.info(i, policy);
                };
                calcSum(expected_results);//期待する実行結果の総計を設定
                that.getErrorMessage();
                if (that.isInvalid) {//エラーがあるかを調べて、モーダルを表示する
                    that.ShowError();//エラーモーダルを表示
                } else {
                    that.BuildTable();//テーブル用のHTMLを生成
                    that.scenario_form.PrepareButtons();//各ボタンの設定する
                    that.ShowTable();//テーブルをDOMへ追加
                    that.scenario_form.PrepareScenarioBoxes(that.passSum + that.dropSum, that.passSum, that.dropSum);//カウント用の表示を初期化
                    that.scenario_form.filter_name = 'all';
                }
                console.log("done");
            }
        }, 1000);
        /**
         * 期待する実行結果の合計を設定する
         * @param {Array} expected_results - 期待する結果が入った配列
         * @inner
         */
        function calcSum(expected_results) {
            that.passSum = $.grep(expected_results, function (n) { return n === "pass" }).length
            that.dropSum = $.grep(expected_results, function (n) { return n === "drop" }).length
        }
        /**
         * 文字コード配列をJavaScript文字列へ変換する。
         * @param {Array} buffer_string - 文字コードが入った配列
         * @inner
         */
        function ConvertString(buffer_string) {
            let buf = new Uint8Array(buffer_string);
            let uniArray = Encoding.convert(buf,{
                to:'UNICODE',
                from:'AUTO',
                bom:true,
                type:'array'
            });
            let out = Encoding.codeToString(uniArray);
            return out;
        }
        /**
         * ファイルの拡張子がcsvか判定する。s
         * @param {string} file_name - 読み込むファイルの名前
         * @return {bool}
         * @inner
         */
        function isCSV(file_name) {
            let extention = file_name.split('.').slice(-1)[0].toLocaleLowerCase();
            if(extention === "csv"){
                return true;
            }
            return false;
        }
    }
    /**
     * ポリシーのエラーメッセージを取得する関数
     * @return {string} エラーメッセージを返す
     */
    Policies.prototype.getErrorMessage = function () {
        let error_msg = "";
        let length = this.policies.length;
        for (let i = 0; i < length; ++i) {
            error_msg += this.policies[i].getMessage(i + 1);//1行目はヘッダーのため(0は飛ばす)
        }
        if (error_msg !== "") this.isInvalid = true;
        this.error_msg = error_msg;
    }
    /**
     * エラーを表示する関数
     */
    Policies.prototype.ShowError = function () {
        $('#modal-scenario-csv-invalid-errors').html(this.error_msg);
        $('#modal-scenario-csv-invalid').css('display', ' block');
    }
    /**
     * テーブル用のHTMLを生成する関数
     */
    Policies.prototype.BuildTable = function () {
        let table_data = "";
        let policy_sum = 0;
        for (let one_policy in this.policies) { // 縦軸のループ
            let policy = this.policies[one_policy];
            policy.BuildTableRow(policy_sum);
            table_data += policy.getTableData();
            policy.BuildURL();
            ++policy_sum;
        }
        this.table_data = table_data;
        this.scenario_form.PrepareScenarioBoxes(this.policies.length, this.passSum, this.dropSum);
    }
    /**
     * テーブルをDOMへ追加する関数
     */
    Policies.prototype.ShowTable = function () {
        $("#scenario-table tbody").append(this.table_data);
    }
    /**
     * シナリオ試験を開始する関数
     */
    Policies.prototype.Start = function () {
        if (this.isPause && this.isStart) {//テスト再開時
            this.ReStart();
        } else {//テスト開始時
            console.info("scenario Start")
            this.SetPolicyNum();
            if (this.sendPolicyArray.length === 0) { return; }
            this.isStart = true;
            this.scenario_form.AllRetry();//全ポリシー再度実行用のフォーム設定
            this.scenario_form.ScenarioStart();
            if (this.IsFilter()) {
              this.DisCountByFilter(); // フィルタ用にカウンタ初期化
            } else {
              this.ClearCount();//内部カウンタを初期化
              this.scenario_form.sendFilter = "all";
            }
            let last = this.sendPolicyArray.length - 1;
            this.SetSendArea(this.sendPolicyArray[0], this.sendPolicyArray[last]);
            this.SendPolicy();
        }
    }
    /**
     * シナリオ試験を一時停止する関数
     */
    Policies.prototype.Pause = function () {
        this.isPause = true;
        this.scenario_form.Pause();
    }
    /**
     * 実際のタイミングの一時停止時に行う処理をまとめたもの
     */
    Policies.prototype.PauseSubRoutine = function () {
        this.scenario_form.status = "Pause";
        this.nextPolicy();
        this.scenario_form.ToggleStartBtn();
        console.info("pause")
    }
    /**
     * シナリオ試験を再開する関数
     */
    Policies.prototype.ReStart = function () {
        console.info("scenario Restart")
        this.isPause = false;
        this.scenario_form.ReStart();
        this.SendPolicy();
    }
    /**
     * 試験範囲(番号)を設定する関数
     * @param {number} begin - ポリシーの開始番号
     * @param {number} end - ポリシーの終了番号
     */
    Policies.prototype.SetSendArea = function (begin, end) {
        if (typeof end === "undefined") { end = this.sendPolicyArray[this.sendPolicyArray.length - 1]; }//endがない場合は最後までを範囲とする。
        this.send_area.begin = begin;
        this.send_area.end = end;
    }
    /**
     * ポリシーの送信をする関数
     * 2017/06/22 フィルター毎の試験実装のため仕様変更
     * 送信対象のポリシーNoを配列に格納して、それをカウントアップする。
     */
    Policies.prototype.SendPolicy = function () {
        this.scenario_form.status = "Sending";
        let that = this;
        console.info("policy No.", this.send_area.begin);

        $('#scenario-table-row' + this.send_area.begin + ' .result').html('<i class="fa fa-spinner fa-pulse"></i>');
        this.policies[this.send_area.begin].Send().then(
            function (data) {
                if (data.Result === 0) { //pass
                    let isMismatch = $('#scenario-table-row' + that.send_area.begin + ' .expected').html() !== 'pass';
                    that.scenario_form.ShowResult(that.send_area.begin, data.Result, isMismatch);//結果表示用のラベルを更新
                    that.policies[that.send_area.begin].field["result"] = 'pass'; //１ポリシーのresult変数に設定
                    ++that.passCount;
                    that.policies[that.send_area.begin].setMismatch(false);
                    if (isMismatch) {// mismatch判定
                        that.policies[that.send_area.begin].setMismatch(true);
                        ++that.mismatchCount;
                    }
                } else if (data.Result === 1) { //drop
                    let isMismatch = $('#scenario-table-row' + that.send_area.begin + ' .expected').html() !== 'drop';
                    that.scenario_form.ShowResult(that.send_area.begin, data.Result,isMismatch);
                    that.policies[that.send_area.begin].field["result"] = 'drop'; //１ポリシーのresult変数に設定
                    that.policies[that.send_area.begin].setMismatch(false);
                    ++that.dropCount;
                    if (isMismatch) { // mismatch判定
                        that.policies[that.send_area.begin].setMismatch(true);
                        ++that.mismatchCount;
                    }
                } else {// バックエンド側のバグで0,1以外が返ってきた場合
                    that.scenario_form.ShowResult(that.send_area.begin, -1);
                    that.policies[that.send_area.begin].field["result"] = 'error'; //１ポリシーのresult変数に設定
                    ++that.errorCount;
                }
                that.UpdateScenarioBoxes();//couterを更新する。
                if (that.send_area.begin === that.send_area.end) { End(); return }//endまで実行すれば終了
                else if (that.isPause) { that.PauseSubRoutine(); }
                else {
                    that.nextPolicy();
                    that.SendPolicy(that.send_area.begin, that.send_area.end); //項目が残っていれば再度実行
                }
            }, function (xhr) {// エラー処理
                //console.log("result: ", xhr);
                that.scenario_form.ShowResult(that.send_area.begin, -1);
                that.policies[that.send_area.begin].field["result"] = 'error';
                ++that.errorCount;
                that.UpdateScenarioBoxes();
                if (that.send_area.begin === that.send_area.end) { End(); return }//endまで実行すれば終了
                else if (that.isPause) { that.PauseSubRoutine(); }
                else {
                    that.nextPolicy();
                    that.SendPolicy(that.send_area.begin, that.send_area.end); //項目が残っていれば再度実行
                }
            });
        /**@inner */
        function End() {//試験終了時の処理
            console.info("scenario End")
            that.scenario_form.ScenarioEnd();
            that.isStart = false;
            that.isPause = false;
            that.scenario_form.status = "";
            that.send_area.begin = 0;
        }
    }
    /**
     * ポリシーテーブルの消去と変数初期化を行う関数
     */
    Policies.prototype.Clear = function () {
        $(".scenario-table-row").removeClass("bg-gray");
        $(".scenario-table-row > .result").html("");
        this.table_data = "";
        this.policies = [];
        this.passSum = 0;
        this.dropSum = 0;
        this.error_msg = "";
        this.isPause = false;
        this.isStart = false;
        this.isInvalid = false;
        this.isFilter = false;
        this.send_area = {};
        this.sendPolicyArray = [];
        this.scenario_form.status = "";
        this.ClearCount();
    }
    /**
     * １ポリシーをリトライする関数
     * @param {number}num リトライするポリシーの番号(0から始まる)
     */
    Policies.prototype.Retry = function (num) {
        if(this.policies[num].field["isMismatch"]){
            this.mismatchCount--;
        }
        switch (this.policies[num].field["result"]) {
            case 'pass':
                this.passCount--;
                this.UpdateScenarioBoxes();
                break;
            case 'drop':
                this.dropCount--;
                this.UpdateScenarioBoxes();
                break;
            case 'error':
                this.errorCount--;
                this.UpdateScenarioBoxes();
                break;
        }
        this.sendPolicyArray = [num];
        this.SetSendArea(num, num);
        this.SendPolicy();
    }
    /**
     * Counterを更新する関数
     */
    Policies.prototype.UpdateScenarioBoxes = function () {
        let allCount = this.passCount + this.dropCount + this.errorCount;//全体のポリシー実行数
        let progress = Math.floor(allCount / (this.passSum + this.dropSum) * 100);//現在の進捗率を計算
        this.scenario_form.UpdateScenarioBoxes(allCount, this.passCount, this.dropCount, this.errorCount + this.mismatchCount, progress);//インジケータを更新
    }
    /**
     * 各結果のカウント変数を初期化する関数
     */
    Policies.prototype.ClearCount = function () {
        this.passCount = 0;
        this.dropCount = 0;
        this.errorCount = 0;
        this.mismatchCount = 0;
        this.sendCount = 0;
    }
    /**
     * フィルター再テスト時のカウント整備
     */
    Policies.prototype.DisCountByFilter = function () {
        let isError = false;
        switch (this.scenario_form.filter_name) {
          case 'pass':
            this.passCount = 0;
            break;
          case 'drop':
            this.dropCount = 0;
            break;
          case 'error':
            this.errorCount = 0;
            isError = true;
            break;
        }
        this.scenario_form.sendFilter = this.scenario_form.filter_name;
        this.DiscountMissmatchDuplicate(isError);
        this.UpdateScenarioBoxes();
        this.sendCount = 0;
    }
    /**
     * ミスマッチによる重複をディスカウントする
     */
    Policies.prototype.DiscountMissmatchDuplicate = function (isError) {
        let danger = $('.disp-tbl-row .result .label-danger');
        let dangerLength = danger.length;
        for (let i = 0; i < dangerLength; i++ ) {
            let currentEle = danger.eq(i);
            let dangerText = currentEle.text();
            let duplicateStatus = "";
            if (dangerText === ' mismatch') {
                this.mismatchCount--;
                if (isError) {
                    duplicateStatus = currentEle.prev().text();
                    switch (duplicateStatus) {
                      case 'pass':
                        this.passCount--;
                        break;
                      case 'drop':
                        this.dropCount--;
                        break;
                    }
                }
            }
        }
    }
    /**
     * 次のポリシーへ
     */
    Policies.prototype.nextPolicy = function () {
        this.send_area.begin = this.sendPolicyArray[++this.sendCount]
    }
    /**
     * フィルタモードチェック
     */
    Policies.prototype.IsFilter = function () {
        return this.isFilter = (this.scenario_form.filter_name !== 'all') ? true : false;
    }
    /**
     * 絞り込んだポリシーの番号を配列に格納する関数
     */
    Policies.prototype.SetPolicyNum = function () {
        this.sendPolicyArray = []; // 初期化
        var that = this;
        $('.disp-tbl-row').each(function () {
            let thisId = $(this).attr("id");
            let p_num  = thisId.match(/\d+/g);
            that.sendPolicyArray.push(parseInt(p_num[0]));
        });
        this.sendPolicyArray = that.sendPolicyArray;
    }
}


/**
 * @class シナリオ画面の入力フォーム部品を操作するためのクラスです。
 * @classdesc 1つの結果分
 * @constructor
 */
var ScenarioForm = function () {
    this.label_name = "";
    this.status = "";
    this.num = 0;
    this.filter_name = "";
    this.sendFilter = ""; // 送信中（一時停止中含む）のフィルタ
    /**
     * シナリオ開始時のフォーム変更
     */
    ScenarioForm.prototype.ScenarioStart = function () {
        $('#scenario-clear-table').attr("disabled", "disabled");
        $('#scenario-test-start').attr("disabled", "disabled");
        $('#scenario-test-pause').removeAttr("disabled", "disabled");
    }
    /**
     * シナリオ終了時のフォーム変更
     */
    ScenarioForm.prototype.ScenarioEnd = function () {
        $('#scenario-test-pause').attr("disabled", "disabled");
        $('#scenario-clear-table').removeAttr("disabled", "disabled");
        $('.scenario-retry-btn').removeAttr("disabled", "disabled");
        $('#scenario-export').removeAttr("disabled", "disabled");
        $('#scenario-test-start').removeAttr("disabled", "disabled");

    }
    /**
     * シナリオ開始時のカウンターの初期化用関数
     */
    ScenarioForm.prototype.PrepareScenarioBoxes = function (all, pass, drop) {
        // Allに反映
        $("#scenario-all-left").html('0');
        $("#scenario-all-right").html(all);
        // Passに反映
        $("#scenario-pass-left").html('0');
        $("#scenario-pass-right").html(pass);
        // Dropに反映
        $("#scenario-drop-left").html('0');
        $("#scenario-drop-right").html(drop);
        // Errorを初期化
        $("#scenario-error").html('0');
        // 選択クラスの削除
        $('#content-scenario .info-box-content').removeClass('selected');
    }
    /**
     * カウンター更新用関数
     * @param {number}  all すべての数
     * @param {number}  passSum passの合計
     * @param {number}  dropSum dropの合計
     * @param {number}  errorSum errorの合計
     * @param {number}  progress 進捗のパーセンテージ
     */
    ScenarioForm.prototype.UpdateScenarioBoxes = function (all, passSum, dropSum, errorSum, progress) {
        $("#scenario-all-left").html(all);
        $("#scenario-pass-left").html(passSum);
        $("#scenario-drop-left").html(dropSum);
        $("#scenario-error").html(errorSum);
        $('#scenario-progress').css('width', progress + '%');
    }
    /**
     * カウンターの初期化用関数
     */
    ScenarioForm.prototype.ClearScenarioBoxes = function () {
        $('#scenario-all-left').text(0);
        $('#scenario-all-right').text(0);
        $('#scenario-pass-left').text(0);
        $('#scenario-pass-right').text(0);
        $('#scenario-drop-left').text(0);
        $('#scenario-drop-right').text(0);
        $("#scenario-error").text(0);
        $('#scenario-progress').css('width', '0%');
    }
    /**
     * 各ボタンの状態初期化用関数
     */
    ScenarioForm.prototype.PrepareButtons = function () {
        $('#scenario-filename').val($('#scenario-upload').val());
        $('#scenario-browse').attr("disabled", "disabled");
        $('#scenario-importfast').css('display', 'none');
        $('#scenario-test-start').removeAttr("disabled", "disabled");
        $('#content-settings', '.form-control').attr("disabled", "disabled");
        $('#content-settings', '.btn').attr("disabled", "disabled");
        $('#one-run').attr("disabled", "disabled");
        $('#check_update').attr("disabled", "disabled");
        $('#scenario-clear-table').removeAttr("disabled", "disabled");
        $('#scenario-table').css('display', 'table');
    }
    /**
     * 試験結果を画面に反映する用の関数
     * @param {number} policy_num 試験番号
     * @param {string} status_code 試験結果
     * @param {bool} [is_mismatch] mismatchの場合
     */
    ScenarioForm.prototype.ShowResult = function (policy_num, status_code, is_mismatch) {
        let label_name = "";
        this.result = "";
        let scenariotest = $('#scenario-table-row' + policy_num).attr("class");

        //リトライ時にclassを初期化する
        if(scenariotest.indexOf("pass") != -1
         || scenariotest.indexOf("drop") != -1
         || scenariotest.indexOf("error") != -1 )
        {
          $('#scenario-table-row' + policy_num).removeClass("pass drop error mismatch");
        }
        switch (status_code) {
            case 0:
                this.result = "pass";
                label_name = "success";
                break;
            case 1:
                this.result = "drop";
                label_name = "warning";
                break;
            case -1:
                this.result = "error";
                label_name = "danger";
                break;
        }
        $('#scenario-table-row' + policy_num).addClass('bg-gray');
        $('#scenario-table-row' + policy_num).addClass(this.result);
        $('#scenario-table-row' + policy_num + ' .result').html('<span class="label label-' + label_name + '">' + this.result + '</span>');
        if (is_mismatch) {
            $('#scenario-table-row' + policy_num).addClass('mismatch');
            $('#scenario-table-row' + policy_num + ' .result').append('<span class="label label-danger"> mismatch</span>');
        }
        //Retryボタンを追加
        $('#scenario-table-row' + policy_num + ' .retry').html('<button type="button" class="btn btn-default btn-xs scenario-retry-btn" disabled onClick="scenario.Retry(' + policy_num + ')">Retry</button> ');
        //時間を追加
        $('#scenario-table-row' + policy_num + ' .time').html(getNow());
        //現在のフィルターと自分の結果が一致すれば表示する。
        if (this.filter_name === this.result || this.filter_name === "all") {
            $('#scenario-table-row' + policy_num).addClass('disp-tbl-row').removeClass('disp-none');
        }
        //結果を表示した位置へ移動
        //this.ScrollTop("scenario-table-row" + policy_num);
    }
    /**
     * 1ポリシーのみリトライする際のフォーム更新用関数
     */
    ScenarioForm.prototype.OneRetry = function (num) {
        $("#scenario-table-row" + num).removeClass("bg-gray");
        $(".retry > button").attr("disabled", "disabled");
    }
    /**
     * 全体のポリシーリトライする用のフォーム更新用関数
     */
    ScenarioForm.prototype.AllRetry = function () {
        $(".disp-tbl-row").removeClass("bg-gray");
        $('.scenario-retry-btn').attr("disabled", "disabled");
        $('#scenario-export').attr("disabled", "disabled");
    }
    /**
     * 試験を一時停止した際のフォーム更新用関数
     */
    ScenarioForm.prototype.Pause = function () {
        $('#scenario-clear-table').removeAttr("disabled", "disabled");
    }
    /**
     * CSV Import時のspinner
     */
    ScenarioForm.prototype.LoadCSV = function () {
        $('#scenario-importfast').html('Loading CSV...\n<i class="fa fa-spinner fa-pulse"></i>');
    }
    /**
     * CSV Import終了
     */
    ScenarioForm.prototype.DefaultMessage = function () {
        $('#scenario-importfast').html('First, Please import a CSV file.');
    }
    /**
     * エラーモーダル表示用の関数
     * @param {string} error_msg エラーメッセージの本文
     */
    ScenarioForm.prototype.ShowError = function (error_msg) {
        $('#modal-scenario-csv-invalid-errors').html(error_msg);
        $('#modal-scenario-csv-invalid').css('display', ' block');
    }
    /**
     * clear policyが押された際のフォーム更新用関数
     */
    ScenarioForm.prototype.ClearPolicies = function () {
        $('#modal-scenario-clear-table').css('display', 'none');
        $('#scenario-table tbody .scenario-table-row').remove();
        $('#scenario-table').css('display', 'none');
        $('#scenario-clear-table').attr("disabled", "disabled");
        $('#scenario-test-start').attr("disabled", "disabled");
        $('#scenario-test-pause').attr("disabled", "disabled");
        $('#scenario-export').attr("disabled", "disabled");
        $('#scenario-importfast').css('display', 'block');
        $('#scenario-browse').removeAttr("disabled", "disabled");
        $('#scenario-upload').val('');
        $('#scenario-filename').val('');
    }
    /**
     * スクロールバーを指定位置に移動する関数
     * @param {number,string} arg numberならばx座標 stringならばセレクタ
     */
    ScenarioForm.prototype.ScrollTop = function (arg) {
        console.log(arg)
        if ($.isNumeric(arg)) {
            $(".wrapper").scrollTop(arg);
            return;
        }
        let position = $(".wrapper").scrollTop() + $("#" + arg).offset().top;
        console.log(position)
        $(".wrapper").scrollTop(position);
    }
    /**
     * 試験結果に応じて表示をフィルタリングする関数
     * @param {string} filter_name フィルタリングの名前
     */
    ScenarioForm.prototype.Filter = function (filter_name, policy_length) {
        let i=0;
        $('#scenario-' + filter_name + '-overlay').css('display', 'block');
        this.filter_name = filter_name;
        let that = this;
        setTimeout(function () {
            switch (filter_name) {
                case "all":
                    $('.scenario-table-row').addClass('disp-tbl-row').removeClass('disp-none');
                    break;
                case "pass":
                    $('.scenario-table-row').addClass('disp-none').removeClass('disp-tbl-row');
                    for (i = 0; i < policy_length; ++i) {
                        $('#scenario-table-row' + i + '.pass').addClass('disp-tbl-row').removeClass('disp-none');
                    }
                    break;
                case "drop":
                    $('.scenario-table-row').addClass('disp-none').removeClass('disp-tbl-row');
                    for (i = 0; i < policy_length; ++i) {
                        $('#scenario-table-row' + i + '.drop').addClass('disp-tbl-row').removeClass('disp-none');
                    }
                    break;
                case "error":
                    $('.scenario-table-row').addClass('disp-none').removeClass('disp-tbl-row');
                    for (i = 0; i < policy_length; ++i) {
                        $('#scenario-table-row' + i + '.error').addClass('disp-tbl-row').removeClass('disp-none');
                        $('#scenario-table-row' + i + '.mismatch').addClass('disp-tbl-row').removeClass('disp-none');
                    }
                    break;
            }
            $('#scenario-' + filter_name + '-overlay').css('display', 'none');
            $('#content-scenario .info-box-content').removeClass('selected');
            $('#scenario-' + filter_name + '-box').addClass('selected');
            that.ToggleStartBtn();
        }, 500);
    }
    /**
     * Pause時に行う関数
     */
    ScenarioForm.prototype.Pause = function () {
        $('#scenario-clear-table').removeAttr("disabled", "disabled");
        $('#scenario-test-pause').attr("disabled", "disabled");
    }
    /**
     * restart時に行う関数
     */
    ScenarioForm.prototype.ReStart = function () {
        $('#scenario-test-start').attr("disabled", "disabled");
        $('#scenario-clear-table').attr("disabled", "disabled");
        $('#scenario-test-pause').removeAttr("disabled", "disabled");
    }
    /**
     * 再生ボタンのdisable切り替え
     */
    ScenarioForm.prototype.ToggleStartBtn = function () {
        let len = $('.disp-tbl-row').length;
        let isDisable = false;
        switch (this.status) {
          case 'Pause':
            isDisable = (this.filter_name !== this.sendFilter) ? true : false;
            break;
          case 'Sending':
            isDisable = true;
            break;
          default:
            isDisable = (len === 0) ? true : false;
            break;
        }
        if (isDisable) {
            $('#scenario-test-start').attr('disabled', 'disabled');
        } else {
            $('#scenario-test-start').removeAttr('disabled', 'disabled');
        }
    }
}


/*
** Click Events
*/
//Scenario ファイル選択時
$('#scenario-browse').click(function () {
    if (needle === null) {
        $("#modal-scenario-unreachable").css('display', 'block');
        $('#modal-scenario-unreachable-close').focus();
    } else {
        $("#modal-scenario-info").css('display', 'block');
        $('#modal-scenario-info-ok').focus();
    }
});
//Scenario confirm message　でokを押した時
$('#modal-scenario-info-ok').click(function () {
    $("#modal-scenario-info").css('display', 'none');
    $('#scenario-upload').click();
});
//Scenario confirm messageの後に必ず呼ばれる。
$('#scenario-upload').change(function () {
    if ($('#scenario-upload').val() === "") {
        return;
    }
    setTimeout(function () {
        scenario.ReadPolicyFile();
    }, 1000);
});
// Clear Policy Tableがクリックされた場合
$('#scenario-clear-table').click(function () {
    $('#modal-scenario-clear-table').css('display', 'block');
});
// Clearを続行した場合
$('#modal-scenario-clear-table-continue').click(function () {
    scenario.ClearPolicies();
});
// NeedleWork本体との通信が取れないモーダルを閉じる時(scenario)
$('#modal-scenario-unreachable-close').click(function () {
    $('#modal-scenario-unreachable').css('display', 'none');
});
// CSV書式異常を示すモーダルを閉じる時
$('#modal-scenario-csv-invalid-close').click(function () {
    $('#scenario-table tbody .scenario-table-row').remove();
    $('#modal-scenario-csv-invalid').css('display', 'none');
    scenario.ClearPolicies();
});
//Test Startをクリック場合
$('#scenario-test-start').click(function () {
    scenario.Start();
})
//pauseボタン
$('#scenario-test-pause').click(function () {
    scenario.Pause();
});
// Display All
$("#scenario-all-box").click(function () {
    scenario.Filter("all");
});
// Filter by Pass
$("#scenario-pass-box").click(function () {
    scenario.Filter("pass");
});
// Filter by Drop
$("#scenario-drop-box").click(function () {
    scenario.Filter("drop");
});
// Filter by Error
$("#scenario-error-box").click(function () {
    scenario.Filter("error");
});
// Clear Policy Tableがクリックされた場合
$('#scenario-clear-table').click(function () {
    $('#modal-scenario-clear-table').css('display', 'block');
});
// Clearを続行した場合
$('#modal-scenario-clear-table-continue').click(function () {
    scenario.ClearPolicies();
});
// Export Test Resultsがクリックされた場合
$('#scenario-export').click(function () {
    scenario.ExportResult();
});
// Scenarioクリック時
$(".sidebar-scenario").click(function () {
    $('.content-wrapper').css('display', 'none');
    $('#content-scenario').css('display', 'block');
    $('.sidebar-menu li').removeClass('active');
    $('.sidebar-menu #menu-scenario').addClass('active');
});
